//---------------------------------------------------------------------------
//
//  Module: vmmwrap.h
//
//  Purpose:
//     Provides necessary macros for the 'C' wrapper.
//
//  Development Team:
//     Bryan A. Woodruff
//
//  History:   Date       Author      Comment
//              3/ 8/93   BryanW      Wrote it.
//
//---------------------------------------------------------------------------
//
//               Copyright (c) 1993 Woodruff Software Systems.
//                          All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef _VMMWRAP_

#pragma pack(1)         /* Assume byte packing throughout */

#ifdef __cplusplus
extern "C" {            /* Assume C declarations for C++ */
#endif   /* __cplusplus */

#define _VMMWRAP_

// data/code segment definitions


#define VxD_LOCKED_CODE_SEG  code_seg( "_LTEXT", "CODE" )
#define VxD_LOCKED_DATA_SEG  data_seg( "_LDATA", "CODE" )
#define VxD_CODE_SEG         code_seg( "_TEXT", "PCODE" )
#define VxD_DATA_SEG         data_seg( "_DATA", "PCODE" )
#define VxD_ICODE_SEG        code_seg( "_ITEXT", "ICODE" )
#define VxD_IDATA_SEG        data_seg( "_IDATA", "ICODE" )
#define VxD_INIT_CODE_SEG VxD_ICODE_SEG
#define VxD_INIT_DATA_SEG VxD_IDATA_SEG
#define VxD_PAGEABLE_CODE_SEG VxD_CODE_SEG
#define VxD_PAGEABLE_DATA_SEG VxD_DATA_SEG



DECLARE_HANDLE( HEVENT ) ;
DECLARE_HANDLE( HLIST ) ;
typedef HLIST *PHLIST ;
DECLARE_HANDLE( HMEM ) ;
typedef HMEM *PHMEM ;
DECLARE_HANDLE( HVM ) ;

#ifndef FIELD_OFFSET
    #define FIELD_OFFSET(type, field)    ((LONG)&(((type *)0)->field))
#endif

//----------------------------------------------------------------------------
//
//                        Device ID equates
//
//----------------------------------------------------------------------------

#define Undefined_Device_ID    0x00000
#define VMM_Device_ID          0x00001   // Used for dynalink table
#define Debug_Device_ID        0x00002
#define VPICD_Device_ID        0x00003
#define VDMAD_Device_ID        0x00004
#define VTD_Device_ID          0x00005
#define V86MMGR_Device_ID      0x00006
#define PageSwap_Device_ID     0x00007
#define Parity_Device_ID       0x00008
#define Reboot_Device_ID       0x00009
#define VDD_Device_ID          0x0000A
#define VSD_Device_ID          0x0000B
#define VMD_Device_ID          0x0000C
#define VKD_Device_ID          0x0000D
#define VCD_Device_ID          0x0000E
#define VPD_Device_ID          0x0000F
#define BlockDev_Device_ID     0x00010
#define VMCPD_Device_ID        0x00011
#define EBIOS_Device_ID        0x00012
#define BIOSXlat_Device_ID     0x00013
#define VNETBIOS_Device_ID     0x00014
#define DOSMGR_Device_ID       0x00015
#define WINLOAD_Device_ID      0x00016
#define SHELL_Device_ID        0x00017
#define VMPoll_Device_ID       0x00018
#define VPROD_Device_ID        0x00019
#define DOSNET_Device_ID       0x0001A
#define VFD_Device_ID          0x0001B
#define VDD2_Device_ID         0x0001C   // Secondary display adapter
#define WINDEBUG_Device_ID     0x0001D
#define TSRLoad_Device_ID      0x0001E   // TSR instance utility ID
#define BiosHook_Device_ID     0x0001F   // Bios interrupt hooker VxD
#define Int13_Device_ID        0x00020
#define PageFile_Device_ID     0x00021   // Paging File device
#define SCSI_Device_ID         0x00022   // SCSI device
#define MCA_POS_Device_ID      0x00023   // MCA_POS device
#define SCSIFD_Device_ID       0x00024   // SCSI FastDisk device
#define VPEND_Device_ID        0x00025   // Pen device
#define APM_Device_ID          0x00026   // Power Management device

//----------------------------------------------------------------------------
//
//                     Initialization order equates
//
//----------------------------------------------------------------------------

#define VMM_Init_Order         0x000000000
#define APM_Init_Order         0x001000000
#define Debug_Init_Order       0x004000000
#define BiosHook_Init_Order    0x006000000
#define VPROD_Init_Order       0x008000000
#define VPICD_Init_Order       0x00C000000
#define VTD_Init_Order         0x014000000
#define PageFile_Init_Order    0x018000000
#define PageSwap_Init_Order    0x01C000000
#define Parity_Init_Order      0x020000000
#define Reboot_Init_Order      0x024000000
#define EBIOS_Init_Order       0x026000000
#define VDD_Init_Order         0x028000000
#define VSD_Init_Order         0x02C000000
#define VCD_Init_Order         0x030000000
#define VMD_Init_Order         0x034000000
#define VKD_Init_Order         0x038000000
#define VPD_Init_Order         0x03C000000
#define BlockDev_Init_Order    0x040000000
#define MCA_POS_Init_Order     0x041000000
#define SCSIFD_Init_Order      0x041400000
#define SCSIMaster_Init_Order  0x041800000
#define Int13_Init_Order       0x042000000
#define VFD_Init_Order         0x044000000
#define VMCPD_Init_Order       0x048000000
#define BIOSXlat_Init_Order    0x050000000
#define VNETBIOS_Init_Order    0x054000000
#define DOSMGR_Init_Order      0x058000000
#define DOSNET_Init_Order      0x05C000000
#define WINLOAD_Init_Order     0x060000000
#define VMPoll_Init_Order      0x064000000

#define Undefined_Init_Order   0x080000000

#define WINDEBUG_Init_Order    0x081000000
#define VDMAD_Init_Order       0x090000000
#define V86MMGR_Init_Order     0x0A0000000

// Device ID for device that must touch memory in 1st MB
// at crit init (after V86mmgr)

#define Undef_Touch_Mem_Init_Order 0x0A8000000
#define SHELL_Init_Order           0x0B0000000

//----------------------------------------------------------------------------
//
//                      System control call messages
//
//----------------------------------------------------------------------------
                            
#define Sys_Critical_Init     0x0000
#define Device_Init           0x0001
#define Init_Complete         0x0002
#define Sys_VM_Init           0x0003
#define Sys_VM_Terminate      0x0004
#define System_Exit           0x0005
#define Sys_Critical_Exit     0x0006
#define Create_VM             0x0007
#define VM_Critical_Init      0x0008
#define VM_Init               0x0009
#define VM_Terminate          0x000A
#define VM_Not_Executable     0x000B
#define Destroy_VM            0x000C
#define VM_Suspend            0x000D
#define VM_Resume             0x000E
#define Set_Device_Focus      0x000F
#define Begin_Message_Mode    0x0010
#define End_Message_Mode      0x0011

#define Reboot_Processor      0x0012
#define Query_Destroy         0x0013
#define Debug_Query           0x0014

#define Begin_PM_App          0x0015

#define BPA_32_Bit            0x0001
#define BPA_32_Bit_Flag       0x0001

#define End_PM_App            0x0016

#define Device_Reboot_Notify  0x0017
#define Crit_Reboot_Notify    0x0018

#define Close_VM_Notify       0x0019

#define CVNF_Crit_Close       0x0001
#define CNVF_Crit_Close_Bit   0x0000

#define Power_Event           0x001A



//----------------------------------------------------------------------------
//
//                    Flags for page allocator calls
//
//----------------------------------------------------------------------------

#define PageZeroInit            0x00000001
#define PageUseAlign            0x00000002
#define PageContig              0x00000004
#define PageFixed               0x00000008
#define PageDEBUGNulFault       0x00000010
#define PageZeroReInit          0x00000020
#define PageNoCopy              0x00000040
#define PageLocked              0x00000080
#define PageLockedIfDP          0x00000100
#define PageSetV86Pageable      0x00000200
#define PageClearV86Pageable    0x00000400
#define PageSetV86IntsLocked    0x00000800
#define PageClearV86IntsLocked  0x00001000
#define PageMarkPageOut         0x00002000
#define PagePDPSetBase          0x00004000
#define PagePDPClearBase        0x00008000
#define PageDiscard             0x00010000
#define PagePDPQueryDirty       0x00020000

// New for 3.10

#define PageMapFreePhysReg      0x00040000

// NOTE: HIGH 8 BITS (bits 24-31) are reserved

//----------------------------------------------------------------------------
//
//                         Page table entry bits
//
//----------------------------------------------------------------------------

#define P_SIZE        0x1000  // page size

#define P_PRES        0x01    // page present bit
#define P_PRESBit     0
#define P_WRITE       0x02    // write access bit
#define P_WRITEBit    1       
#define P_USER        0x04    // access bit for User mode
#define P_USERBit     2       
#define P_ACC         0x20    // page accessed bit
#define P_ACCBit      5       
#define P_DIRTY       0x40    // page dirty bit
#define P_DIRTYBit    6

#define P_AVAIL       (P_PRES|P_WRITE|P_USER) // avail to everyone & present

//---------------------------------------------------
//
// Page types - definition of the OS reserved
//              bits in the page table entry.
//
//---------------------------------------------------

#define PG_TYPE       0x0E00  // TYPE bits in PTE

//---------------------------------------------------
//
//   Page types for page allocator calls
//
//---------------------------------------------------

#define PG_VM         0
#define PG_SYS        1
#define PG_RESERVED1  2
#define PG_PRIVATE    3
#define PG_RESERVED2  4
#define PG_RELOCK     5      // PRIVATE to MMGR
#define PG_INSTANCE   6
#define PG_HOOKED     7
#define PG_IGNORE     0x0FFFFFFFF

//---------------------------------------------------
//
//   Types for page table entries
//
//---------------------------------------------------

#define PgT_VM        (PG_VM << 9)
#define PgT_SYS       (PG_SYS << 9)
#define PgT_RESERVED1 (PG_RESERVED1 << 9)
#define PgT_PRIVATE   (PG_PRIVATE << 9)
#define PgT_RESERVED2 (PG_RESERVED2 << 9)
#define PgT_RELOCK    (PG_RELOCK << 9)
#define PgT_INSTANCE  (PG_INSTANCE << 9)
#define PgT_HOOKED    (PG_HOOKED << 9)

//----------------------------------------------------------------------------
//
//            Definitions for the access byte in a descriptor
//
//----------------------------------------------------------------------------

#define D_PRES       0x80     // present in memory
#define D_NOTPRES    0x00     // not present in memory

#define D_DPL0       0x00     // Ring 0
#define D_DPL1       0x20     // Ring 1
#define D_DPL2       0x40     // Ring 2
#define D_DPL3       0x60     // Ring 3

#define D_SEG        0x10     // Segment descriptor
#define D_CTRL       0x00     // Control descriptor

#define D_GRAN_BYTE  0x00     // Segment length is byte granular
#define D_GRAN_PAGE  0x80     // Segment length is page granular
#define D_DEF16      0x00     // Default operation size is 16 bits
#define D_DEF32      0x40     // Default operation size is 32 bits


#define D_CODE       0x08     // code
#define D_DATA       0x00     // data

#define D_RX         0x02     // if code, readable
#define D_X          0x00     // if code, exec only
#define D_W          0x02     // if data, writable
#define D_R          0x00     // if data, read only

#define D_ACCESSED   0x01     // segment accessed bit

#define RW_Data_Type (D_PRES+D_SEG+D_DATA+D_W)
#define R_Data_Type  (D_PRES+D_SEG+D_DATA+D_R)
#define Code_Type    (D_PRES+D_SEG+D_CODE+D_RX)

#define D_PAGE32     (D_GRAN_PAGE+D_DEF32)

#define SELECTOR_MASK  0xfff8      // selector index
#define SEL_LOW_MASK   0x00f8      // mask for low byte of sel indx
#define TABLE_MASK     0x0004      // table bit
#define RPL_MASK       0x0003      // priviledge bits
#define RPL_CLR        (~0x0003)   // clear ring bits

//----------------------------------------------------------------------------
//
//             Flags for _Allocate_Global_V86_Data_Area
//
//----------------------------------------------------------------------------

#define GVDAWordAlign      0x0001
#define GVDADWordAlign     0x0002
#define GVDAParaAlign      0x0004
#define GVDAPageAlign      0x0008
#define GVDAInstance       0x0100
#define GVDAZeroInit       0x0200
#define GVDAReclaim        0x0400

// New for 3.10

#define GVDAInquire        0x0800
#define GVDAHighSysCritOK  0x1000

//----------------------------------------------------------------------------
//
//                     Flags for heap allocator calls
//
//----------------------------------------------------------------------------

#define HeapZeroInit    0x0001
#define HeapZeroReInit  0x0002
#define HeapNoCopy      0x0004

//----------------------------------------------------------------------------
//
//                     Flags for List_Create
//
//----------------------------------------------------------------------------

#define LF_Async            0x0001
#define LF_Async_Bit        0
#define LF_Use_Heap         0x0002
#define LF_Use_Heap_Bit     1
#define LF_Alloc_Error      0x0004
#define LF_Alloc_Error_Bit  2

//----------------------------------------------------------------------------
//
//                         Scheduler Boost Values
//
//----------------------------------------------------------------------------

#define Reserved_Low_Boost      0x00000001
#define Cur_Run_VM_Boost        0x00000004
#define Low_Pri_Device_Boost    0x00000010
#define High_Pri_Device_Boost   0x00001000
#define Critical_Section_Boost  0x00100000
#define Time_Critical_Boost     0x00400000
#define Reserved_High_Boost     0x40000000

//----------------------------------------------------------------------------
//
//               Flags for Call_Priority_VM_Event
//
//----------------------------------------------------------------------------

#define PEF_Wait_For_STI       0x0001
#define PEF_Wait_For_STI_Bit   0
#define PEF_Wait_Not_Crit      0x0002
#define PEF_Wait_Not_Crit_Bit  1
#define PEF_Dont_Unboost       0x0004
#define PEF_Dont_Unboost_Bit   2
#define PEF_Always_Sched       0x0008
#define PEF_Always_Sched_Bit   3
#define PEF_Time_Out           0x0010
#define PEF_Time_Out_Bit       4

//----------------------------------------------------------------------------
//
//        Flags for Begin_Critical_Section and Wait_Semaphore
//
//----------------------------------------------------------------------------

#define Block_Svc_Ints                0x0001
#define Block_Svc_Ints_Bit            0
#define Block_Svc_If_Ints_Locked      0x0002
#define Block_Svc_If_Ints_Locked_Bit  1
#define Block_Enable_Ints             0x0004
#define Block_Enable_Ints_Bit         2
#define Block_Poll                    0x0008
#define Block_Poll_Bit                3

//----------------------------------------------------------------------------
//
//                            Data structures
//
//----------------------------------------------------------------------------

typedef struct tagREGS
{
    DWORD  eflags ;
    DWORD  edi ;
    DWORD  esi ;
    DWORD  ebp ;
    DWORD  esp ;
    DWORD  ebx ;
    DWORD  edx ;
    DWORD  ecx ;
    DWORD  eax ;

} REGS, *PREGS ;

#define EFLAGS_CF   0x00000001
#define EFLAGS_PF   0x00000004
#define EFLAGS_AF   0x00000010
#define EFLAGS_ZF   0x00000040
#define EFLAGS_SF   0x00000080

typedef struct tagCRS_32
{
   DWORD  Client_EDI ;
   DWORD  Client_ESI ;
   DWORD  Client_EBP ;
   DWORD  dwReserved_1 ;          // ESP at pushall
   DWORD  Client_EBX ;
   DWORD  Client_EDX ;
   DWORD  Client_ECX ;
   DWORD  Client_EAX ;
   DWORD  Client_Error ;          // DWORD error code
   DWORD  Client_EIP ;
   WORD   Client_CS ;
   WORD   wReserved_2 ;           // (padding)
   DWORD  Client_EFlags ;
   DWORD  Client_ESP ;
   WORD   Client_SS ;
   WORD   wReserved_3 ;           // (padding)
   WORD   Client_ES ;
   WORD   WReserved_4 ;           // (padding)
   WORD   Client_DS ;
   WORD   wReserved_5 ;           // (padding)
   WORD   Client_FS ;
   WORD   wReserved_6 ;           // (padding)
   WORD   Client_GS ;
   WORD   wReserved_7 ;           // (padding)

   DWORD  Client_Alt_EIP ;
   WORD   Client_Alt_CS ;
   WORD   wReserved_8 ;           // (padding)
   DWORD  Client_Alt_EFlags ;
   DWORD  Client_Alt_ESP ;
   WORD   Client_Alt_SS ;
   WORD   wReserved_9 ;           // (padding)
   WORD   Client_Alt_ES ;
   WORD   WReserved_10 ;          // (padding)
   WORD   Client_Alt_DS ;
   WORD   wReserved_11 ;          // (padding)
   WORD   Client_Alt_FS ;
   WORD   wReserved_12 ;          // (padding)
   WORD   Client_Alt_GS ;
   WORD   wReserved_13 ;          // (padding)

} CRS_32, *PCRS_32 ;

typedef struct tagCRS_16
{
   WORD   Client_DI ;
   WORD   wReserved_1 ;           // (padding)
   WORD   Client_SI ;
   WORD   wReserved_2 ;           // (padding)
   WORD   Client_BP ;
   WORD   wReserved_3 ;           // (padding)
   DWORD  dwReserved_4 ;          // ESP at pushall
   WORD   Client_BX ;
   WORD   wReserved_5 ;           // (padding)
   WORD   Client_DX ;
   WORD   wReserved_6 ;           // (padding)
   WORD   Client_CX ;
   WORD   wReserved_7 ;           // (padding)
   WORD   Client_AX ;
   WORD   wReserved_8 ;           // (padding)
   DWORD  dwReserved_9 ;          // DWORD error code
   WORD   Client_IP ;
   WORD   wReserved_10 ;          // (padding)
   WORD   Client_CS ;
   WORD   wReserved_11 ;          // (padding)
   WORD   Client_Flags ;          // flags (loword)
   WORD   wReserved_12 ;          // (padding)
   WORD   Client_SP ;
   WORD   wReserved_13 ;          // (padding)
   WORD   Client_SS ;
   WORD   wReserved_14 ;          // (padding)
   WORD   Client_ES ;
   WORD   wReserved_15 ;          // (padding)
   WORD   Client_DS ;
   WORD   wReserved_16 ;          // (padding)
   WORD   Client_FS ;
   WORD   wReserved_17 ;          // (padding)
   WORD   Client_GS ;
   WORD   wReserved_18 ;          // (padding)

   WORD   Client_Alt_IP ;
   WORD   wReserved_19 ;          // (padding)
   WORD   Client_Alt_CS ;
   WORD   wReserved_20 ;          // (padding)
   WORD   Client_Alt_Flags ;
   WORD   wReserved_21 ;          // (padding)
   WORD   Client_Alt_SP ;
   WORD   wReserved_22 ;          // (padding)
   WORD   Client_Alt_SS ;
   WORD   wReserved_23 ;          // (padding)
   WORD   Client_Alt_ES ;
   WORD   wReserved_24 ;          // (padding)
   WORD   Client_Alt_DS ;
   WORD   wReserved_25 ;          // (padding)
   WORD   Client_Alt_FS ;
   WORD   wReserved_26 ;          // (padding)
   WORD   Client_Alt_GS ;
   WORD   wReserved_27 ;          // (padding)

} CRS_16, *PCRS_16 ;

typedef struct tagCRS_8
{
   WORD   Client_DI ;
   WORD   wReserved_1 ;           // (padding)
   WORD   Client_SI ;
   WORD   wReserved_2 ;           // (padding)
   WORD   Client_BP ;
   WORD   wReserved_3 ;           // (padding)
   DWORD  dwReserved_4 ;          // ESP at pushall
   BYTE   Client_BL ;
   BYTE   Client_BH ;
   WORD   wReserved_5 ;           // (padding)
   BYTE   Client_DL ;
   BYTE   Client_DH ;
   WORD   wReserved_6 ;           // (padding)
   BYTE   Client_CL ;
   BYTE   Client_CH ;
   WORD   wReserved_7 ;           // (padding)
   BYTE   Client_AL ;
   BYTE   Client_AH ;
   WORD   wReserved_8 ;           // (padding)
   DWORD  dwReserved_9 ;          // DWORD error code
   WORD   Client_IP ;
   WORD   wReserved_10 ;          // (padding)
   WORD   Client_CS ;
   WORD   wReserved_11 ;          // (padding)
   WORD   Client_Flags ;          // flags (loword)
   WORD   wReserved_12 ;          // (padding)
   WORD   Client_SP ;
   WORD   wReserved_13 ;          // (padding)
   WORD   Client_SS ;
   WORD   wReserved_14 ;          // (padding)
   WORD   Client_ES ;
   WORD   wReserved_15 ;          // (padding)
   WORD   Client_DS ;
   WORD   wReserved_16 ;          // (padding)
   WORD   Client_FS ;
   WORD   wReserved_17 ;          // (padding)
   WORD   Client_GS ;
   WORD   wReserved_18 ;          // (padding)

   WORD   Client_Alt_IP ;
   WORD   wReserved_19 ;          // (padding)
   WORD   Client_Alt_CS ;
   WORD   wReserved_20 ;          // (padding)
   WORD   Client_Alt_Flags ;
   WORD   wReserved_21 ;          // (padding)
   WORD   Client_Alt_SP ;
   WORD   wReserved_22 ;          // (padding)
   WORD   Client_Alt_SS ;
   WORD   wReserved_23 ;          // (padding)
   WORD   Client_Alt_ES ;
   WORD   wReserved_24 ;          // (padding)
   WORD   Client_Alt_DS ;
   WORD   wReserved_25 ;          // (padding)
   WORD   Client_Alt_FS ;
   WORD   wReserved_26 ;          // (padding)
   WORD   Client_Alt_GS ;
   WORD   wReserved_27 ;          // (padding)

} CRS_8, *PCRS_8 ;

//----------------------------------------------------------------------------
//                VM control block structure (VMM)
//----------------------------------------------------------------------------

typedef struct tagVMMCB
{
   DWORD  CB_VM_Status ;
   DWORD  CB_High_Linear ;
   DWORD  CB_Client_Pointer ;
   DWORD  CB_VMID ;

} VMMCB, *PVMMCB ;

// VM status indicates globally interesting VM states

#define VMStat_Exclusive           0x00001 // VM is exclusive mode
#define VMStat_Exclusive_Bit       0x00
#define VMStat_Background          0x00002 // VM runs in background
#define VMStat_Background_Bit      0x01
#define VMStat_Creating            0x00004 // In process of creating
#define VMStat_Creating_Bit        0x02
#define VMStat_Suspended           0x00008 // VM not scheduled
#define VMStat_Suspended_Bit       0x03
#define VMStat_Not_Executeable     0x00010 // VM partially destroyed
#define VMStat_Not_Executeable_Bit 0x04
#define VMStat_PM_Exec             0x00020 // Currently in PM app
#define VMStat_PM_Exec_Bit         0x05
#define VMStat_PM_App              0x00040 // PM app present in VM
#define VMStat_PM_App_Bit          0x06
#define VMStat_PM_Use32            0x00080 // PM app is 32-bit
#define VMStat_PM_Use32_Bit        0x07
#define VMStat_VxD_Exec            0x00100 // Call from VxD
#define VMStat_VxD_Exec_Bit        0x08
#define VMStat_High_Pri_Back       0x00200 // High pri background
#define VMStat_High_Pri_Back_Bit   0x09
#define VMStat_Blocked             0x00400 // Blocked on semaphore
#define VMStat_Blocked_Bit         0x0A
#define VMStat_Awakening           0x00800 // Woke up after blocked
#define VMStat_Awakening_Bit       0x0B
#define VMStat_PageableV86         0x01000 // part of V86 is pageable (PM app)
#define VMStat_PageableV86Bit      0x0C
#define VMStat_V86IntsLocked       0x02000 // Rest of V86 is locked
#define VMStat_V86IntsLockedBit    0x0D    //    regardless of pager type
#define VMStat_TS_Sched            0x04000 // Scheduled by time-slicer
#define VMStat_TS_Sched_Bit        0x0E
#define VMStat_Idle                0x08000 // VM has released time slice
#define VMStat_Idle_Bit            0x0F
#define VMStat_Closing             0x10000 // Close_VM called for VM
#define VMStat_Closing_Bit         0x10

#define VMStat_Use32_Mask          (VMStat_PM_Use32 | VMStat_VxD_Exec)

//----------------------------------------------------------------------------
//                Device descriptor block structure
//----------------------------------------------------------------------------

#define DDK_Version 0x30A

// data type for API procedures

typedef VOID (CDECL* APIPROC)( HVM, PCRS_32 ) ;

typedef struct tagVxD_Desc_Block
{
   DWORD    DDB_Next ;                // VMM reserved field
   WORD     DDB_SDK_Version  ;        // VMM reserved field
   WORD     DDB_Req_Device_Number ;   // Required device number
   BYTE     DDB_Dev_Major_Version ;   // Major device number
   BYTE     DDB_Dev_Minor_Version ;   // Minor device number
   WORD     DDB_Flags ;               // Flags for init calls complete
   BYTE     DDB_Name[ 8 ] ;           // Device name
   DWORD    DDB_Init_Order ;          // Initialization Order
   DWORD    DDB_Control_Proc ;        // Offset of control procedure
   APIPROC  DDB_V86_API_Proc ;        // Offset of API procedure
   APIPROC  DDB_PM_API_Proc ;         // Offset of API procedure
   DWORD    DDB_V86_API_CSIP ;        // CS:IP of API entry point
   DWORD    DDB_PM_API_CSIP ;         // CS:IP of API entry point
   DWORD    DDB_Reference_Data ;      // Reference data from real mode
   DWORD    DDB_Service_Table_Ptr ;   // Pointer to service table
   DWORD    DDB_Service_Table_Size ;  // Number of services


} DDB ;

//----------------------------------------------------------------------------
//               Data structures for VxD service tables
//----------------------------------------------------------------------------

typedef VOID (CDECL* SERVICEPROC)( PREGS ) ;

//----------------------------------------------------------------------------
//             Data structures for vmmInstallMultIOHandlers
//----------------------------------------------------------------------------

typedef UINT (CDECL* VXDIOPROC)( DWORD, UINT, UINT, PCRS_32, UINT ) ;

typedef struct tagVXDIOENTRY
{
   WORD       VxD_IO_Port ;
   VXDIOPROC  VxD_IO_Proc ;

} VXDIOENTRY, *PVXDIOENTRY ;

#ifndef __cplusplus

typedef struct tagVXDIOTABLE
{
   WORD        IOT_Ports ;
   VXDIOENTRY  IOT_Entries[] ;

} VXDIOTABLE, *PVXDIOTABLE ;

#else

typedef struct tagVXDIOTABLE
{
   WORD        IOT_Ports ;
   VXDIOENTRY  IOT_Entries ;

} VXDIOTABLE, *PVXDIOTABLE ;

#define IOTABLE_Entry (&IOT_Entries)

#endif

//----------------------------------------------------------------------------
//                Data structure for vmmAddInstanceItem
//----------------------------------------------------------------------------

typedef struct tagINSTDATASTRUC
{
   DWORD   dwInstLinkF ;      // reserved set to NULL
   DWORD   dwInstLinkB ;      // reserved set to NULL
   DWORD   dwInstLinAddr ;    // linear address of start of block
   DWORD   dwInstSize ;       // size of block in bytes
   DWORD   dwInstType ;       // type of block

} INSTDATASTRUC, *PINSTDATASTRUC ;

// values for dwInstType

#define INDOS_Field   0x0100
#define ALWAYS_Field  0x0200

//----------------------------------------------------------------------------
//             Data structure for Instance Page Fault handlers
//----------------------------------------------------------------------------

typedef struct tagIPFDATA
{

   DWORD  IPF_LinAddr ;       // CR2 address of fault
   DWORD  IPF_MapPageNum ;    // Possible converted page # of fault
   DWORD  IPF_PTEEntry ;      // Contents of PTE that faulted
   DWORD  IPF_FaultingVM ;    // May not = Current VM (IPF_V86PgH set)
   DWORD  IPF_Flags ;         // Flags

} IPFDATA, *PIPFDATA ;

// data types for message dispatch table used by VMMWRAP.ASM

typedef struct tagDISPATCHINFO
{
   DWORD  dwMsg ;
   PVOID  pProc ;

} DISPATCHINFO ;

// data type for v86 interrupt callbacks

typedef BOOL (CDECL* V86INTPROC)( HVM, UINT, PCRS_32 ) ;

// data type for v86 page hook

typedef VOID (CDECL* V86PHPROC)( DWORD, UINT ) ;

// data type for invalid page fault callbacks

typedef BOOL (CDECL* IPFPROC)( DWORD, PIPFDATA ) ;

// data type for event/generic callbacks

typedef VOID (CDECL* VXDCBPROC)( HVM, DWORD, PCRS_32 ) ;

// function prototypes

VOID CDECL vmmwrapVxDControlProc( VOID ) ;
DWORD CDECL vmmwrapThunkVDMADProc( DWORD ) ;
VXDIOPROC CDECL vmmwrapThunkVXDIOProc( VXDIOPROC ) ;
V86INTPROC CDECL vmmwrapThunkV86IntProc( V86INTPROC ) ;
V86PHPROC CDECL vmmwrapThunkV86PHProc( V86PHPROC ) ;
IPFPROC CDECL vmmwrapThunkIPFProc( IPFPROC ) ;
VXDCBPROC CDECL vmmwrapThunkVxDCBProc( VXDCBPROC ) ;
APIPROC CDECL vmmwrapThunkAPIProc( APIPROC ) ;
VOID CDECL vmmwrapCriticalSection( DWORD ) ;
#define CS_BEGIN 0
#define CS_END   1

#ifdef DEBUG
  #ifdef __WATCOMC__
    VOID vmmwrapDebugBreak( VOID ) ;
    #pragma aux vmmwrapDebugBreak = "int 1" ;
  #else
    #define vmmwrapDebugBreak()    _asm int 1
  #endif
#else
    #define vmmwrapDebugBreak()
#endif

UINT CDECL vmmGetVMMVersion( VOID ) ;
UINT CDECL vmmGetCurVMHandle( VOID ) ;
BOOL CDECL vmmTestCurVMHandle( UINT ) ;
UINT CDECL vmmGetSysVMHandle( VOID ) ;
BOOL CDECL vmmTestSysVMHandle( UINT ) ;
BOOL CDECL vmmValidateVMHandle( UINT ) ;

UINT CDECL vmmGetLastUpdatedSystemTime( VOID ) ;

VOID CDECL vmmBeginCriticalSection( DWORD, UINT ) ;
VOID CDECL vmmEndCriticalSection( DWORD ) ;

DWORD CDECL vmmAllocatePMCallBack
(
    VXDCBPROC       fnCallBack,
    DWORD           dwRefData
) ;

VOID CDECL vmmGetV86IntVector( UINT, PWORD, PWORD ) ;
VOID CDECL vmmSetV86IntVector( UINT, DWORD, DWORD ) ;

DWORD CDECL vmmAllocateV86CallBack( DWORD, VXDCBPROC ) ; 

UINT CDECL vmmAllocateDeviceCBArea( UINT, UINT ) ;
BOOL CDECL vmmAddInstanceItem( PINSTDATASTRUC, UINT ) ;
VOID CDECL vmmBuildDescriptorDWORDs( PDWORD, PDWORD, PSTR, UINT,
                                      UINT, UINT, UINT ) ;
UINT CDECL vmmAllocateGDTSelector( DWORD, DWORD, UINT ) ;
UINT CDECL vmmMapPhysToLinear( DWORD, UINT, UINT ) ;
BOOL CDECL vmmPhysIntoV86( DWORD, DWORD, DWORD, UINT, UINT ) ;
PVOID CDECL vmmPageAllocate( PHMEM, UINT, UINT, HVM, UINT, UINT, UINT,
                             PVOID *, UINT ) ;
PVOID CDECL vmmPageReAllocate( PHMEM, HMEM, UINT, UINT ) ;
BOOL CDECL vmmPageFree( HMEM, UINT ) ;
BOOL CDECL vmmPageLock( HMEM, UINT, UINT, UINT ) ;
BOOL CDECL vmmPageUnLock( HMEM, UINT, UINT, UINT ) ;

BOOL CDECL vmmLinPageLock( UINT, UINT, DWORD ) ;
BOOL CDECL vmmLinPageUnLock( UINT, UINT, DWORD ) ;

PVOID CDECL vmmHeapAllocate( DWORD, DWORD ) ;
ULONG CDECL vmmHeapGetSize( PVOID, DWORD ) ;
PVOID CDECL vmmHeapReAllocate( PVOID, DWORD, DWORD ) ;
BOOL CDECL vmmHeapFree( PVOID, DWORD ) ;

BOOL CDECL vmmCopyPageTable( UINT, UINT, PDWORD, UINT ) ;

UINT CDECL vmmGetFirstV86Page( DWORD ) ;
UINT CDECL vmmGetLastV86Page( DWORD ) ;
BOOL CDECL vmmSetLastV86Page( UINT, DWORD ) ;

PSTR CDECL vmmSelectorMapFlat( HVM, DWORD, DWORD ) ;
PSTR CDECL vmmMapFlat( HVM, PCRS_32, DWORD, DWORD ) ;

BOOL CDECL vmmGetDeviceV86PagesArray( DWORD, PDWORD, UINT ) ;
BOOL CDECL vmmAssignDeviceV86Pages( UINT, UINT, DWORD, UINT ) ;

BOOL CDECL vmmTestDebugInstalled( VOID ) ;
VOID CDECL vmmDebugOut( PSTR ) ;
VOID CDECL vmmTraceOut( PSTR ) ;
VOID CDECL vmmTraceOutParam( PSTR, DWORD ) ;

HEVENT vmmCallPriorityVMEvent( HVM, DWORD, DWORD, DWORD, VXDCBPROC, DWORD ) ;

BOOL CDECL vmmInstallMultIOHandlers( PVXDIOTABLE ) ;
VOID CDECL vmmEnableLocalTrapping( DWORD, UINT ) ;
VOID CDECL vmmDisableLocalTrapping( DWORD, UINT ) ;

PVOID CDECL vmmListAllocate( HLIST ) ;
VOID CDECL vmmListDeallocate( HLIST, PVOID ) ;
VOID CDECL vmmListAttach( HLIST, PVOID ) ;
VOID CDECL vmmListAttachTail( HLIST, PVOID ) ;
HLIST CDECL vmmListCreate( DWORD fdwAlloc, DWORD cbNodeSize ) ;
VOID CDECL vmmListDestroy( HLIST ) ;
PVOID CDECL vmmListGetFirst( HLIST ) ;
PVOID CDECL vmmListGetNext( HLIST, PVOID ) ;
PVOID CDECL vmmListRemoveFirst( HLIST ) ;
BOOL CDECL vmmListRemove( HLIST, PVOID ) ;

UINT CDECL vmmGetExecPath( PSTR ) ;

VOID CDECL vmmSaveClientState( PCRS_32 ) ;
VOID CDECL vmmRestoreClientState( PCRS_32 ) ;

VOID CDECL vmmBeginNestExec( VOID ) ;
VOID CDECL vmmBeginNestV86Exec( VOID ) ;
VOID CDECL vmmEndNestExec( VOID ) ;
VOID CDECL vmmResumeExec( VOID ) ;

VOID CDECL vmmExecInt( UINT ) ;
VOID CDECL vmmExecVxDInt( UINT ) ;


VOID CDECL vmmSimulatePush( DWORD ) ;
DWORD CDECL vmmSimulatePop( VOID ) ;
VOID CDECL vmmSimulateFarCall( DWORD, DWORD ) ;
VOID CDECL vmmSimulateFarJmp( DWORD, DWORD ) ;
VOID CDECL vmmSimulateIret( VOID ) ;
VOID CDECL vmmSimulateFarRet( VOID ) ;

PVOID CDECL vmmAllocateTempV86DataArea( ULONG, DWORD ) ;
BOOL CDECL vmmFreeTempV86DataArea( VOID ) ;
DWORD CDECL vmmGetNulPageHandle( VOID ) ;
BOOL CDECL vmmMapIntoV86( DWORD, DWORD, UINT, UINT, UINT, UINT ) ;
BOOL CDECL vmmHookInvalidPageFault( IPFPROC ) ;
BOOL CDECL vmmHookV86IntChain( UINT, V86INTPROC ) ;
BOOL CDECL vmmHookV86Page( UINT, V86PHPROC ) ;
BOOL CDECL vmmModifyPageBits( DWORD, UINT, UINT, DWORD, DWORD,
                              DWORD, DWORD ) ;

DWORD CDECL vmmAllocateGlobalV86DataArea( UINT, UINT ) ;

#define GPE_NOKEY    -1
#define GPE_NOVALUE  -2

BOOL CDECL vmmGetProfileBoolean( PSTR, PSTR, BOOL, PINT ) ;
UINT CDECL vmmGetProfileHexInt( PSTR, PSTR, UINT, PINT ) ;
UINT CDECL vmmGetProfileDecimalInt( PSTR, PSTR, UINT, PINT ) ;

DECLARE_HANDLE( HSEM ) ;
HSEM CDECL vmmCreateSemaphore( DWORD ) ;
VOID CDECL vmmWaitSemaphore( HSEM, DWORD ) ;
VOID CDECL vmmSignalSemaphore( HSEM ) ;
VOID CDECL vmmDestroySemaphore( HSEM ) ;

// DOSMGR services...

BOOL CDECL dosmgrAddDevice
(
    HVM             hVM,
    PVOID           pDevHdr
) ;

// V86MMGR services...

typedef struct tagV86MMGRXLATBUFFSTATE
{
   DWORD            cbAvail ;
   DWORD            cbTotal ;
   WORD             wXlatSeg ;

} V86MMGRXLATBUFFSTATE, *PV86MMGRXLATBUFFSTATE ;

VOID CDECL v86mmgrGetXlatBuffState
(
    HVM                     hVM,
    PV86MMGRXLATBUFFSTATE   pvxbs
) ;

VOID CDECL v86mmgrSetXlatBuffState
(
    HVM                     hVM,
    PV86MMGRXLATBUFFSTATE   pvxbs
) ;


#ifdef __cplusplus
}                       /* End of extern "C" { */
#endif   /* __cplusplus */

#endif

//---------------------------------------------------------------------------
//  End of File: vmmwrap.h
//---------------------------------------------------------------------------

