        page    60, 132

;******************************************************************************
	title	VMMWRAP - VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    A 'C' wrapper for VMM services/control
;
;   Module:   Critical Sections
;
;   Version:  2.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE SegDefs.Inc
        .LIST

VxD_LOCKED_DATA_SEG

;
; vmmwrapCriticalSection() stuff..
;

CS_BEGIN        equ     0
CS_END          equ     1

dwCSCount       dd      0
dwCSIntsEnabled dd      0

VxD_LOCKED_DATA_ENDS

VxD_LOCKED_CODE_SEG

;------------------------------------------------------------------------------
;
;   VOID CDECL _vmmwrapCriticalSection( DWORD dwFlags )
;
;   DESCRIPTION:
;      Disables/enables interrupts and maintains a nest count.
;      This should not be confused with the Critical_Section services
;      of VMM.
;
;   PARAMETERS:
;      DWORD dwFlags
;         CS_BEGIN to begin critical section
;         CS_END to end critical section
;
;   RETURN VALUE:
;      Nothing.
;
;------------------------------------------------------------------------------

BeginProc _vmmwrapCriticalSection, PUBLIC
        
        dwFlags equ     dword ptr [ebp + 8]

        push    ebp
        mov     ebp, esp

        cmp     dwFlags, CS_END
        jne     SHORT VCS_BeginCS

        cmp     dwCSCount, 0
        je      SHORT VCS_Exit
        dec     dwCSCount
        jnz     SHORT VCS_Exit
        cmp     dwCSIntsEnabled, 1
        je      SHORT VCS_Exit

        sti
        mov     dwCSIntsEnabled, 1
        jmp     SHORT VCS_Exit

VCS_BeginCS:
        cmp     dwCSCount, 0
        jne     SHORT VCS_JustInc

        mov     dwCSIntsEnabled, 0
        pushfd
        pop     eax
        test    ah, 2
        jz      SHORT VCS_JustInc
        cli

VCS_JustInc:
        inc     dwCSCount

VCS_Exit:
        pop     ebp
        ret

EndProc _vmmwrapCriticalSection

VxD_LOCKED_CODE_ENDS

END

