        page    60, 132

;******************************************************************************
	title   VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VMMWRAP - A 'C' wrapper for VMM services/control
;
;   Module:   Thunks
;
;   Version:  2.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE SegDefs.Inc
        .LIST

EXTRN Allocate_Procedure_Thunk:NEAR

VxD_PAGEABLE_CODE_SEG

;------------------------------------------------------------------------------
;
;   APIPROC CDECL vmmwrapThunkAPIProc( APIPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for VxD API procedures.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            VOID CDECL APIProc( DWORD hVM, PCRS_32 pCRS )
;
;   RETURN VALUE:
;      APIPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc _vmmwrapThunkAPIProc, PUBLIC
        
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VAProc_Failure
        jmp     SHORT VAProc_CreateThunk

;==================
; Begin thunk code

APIThunk label   byte

        push    ebp
        push    ebx                             ; hVM

        call    $
APIThunkCallAddr equ $-APIThunk
        add     esp, 8                          ; fixup for CDECL
        ret

APIThunkSize    equ     $-APIThunk

;
;End thunk code
;==================

VAProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, APIThunk
        mov     edi, eax
        mov     ecx, APIThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, APIThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VAProc_Exit


VAProc_Failure:
        xor     eax, eax

VAProc_Exit:
        pop     ebp
        ret

EndProc _vmmwrapThunkAPIProc

VxD_PAGEABLE_CODE_ENDS

END

