        page    60, 132

;******************************************************************************
	title   VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VMMWRAP - A 'C' wrapper for VMM services/control
;
;   Module:   Thunks
;
;   Version:  2.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE SegDefs.Inc
        .LIST

EXTRN Allocate_Procedure_Thunk:NEAR

VxD_PAGEABLE_CODE_SEG

;------------------------------------------------------------------------------
;
;   DWORD CDECL vmmwrapThunkVDMADProc( DWORD pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for Virtualized DMA callback procedures.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            BOOL CDECL VDMADProc( DWORD hVM, UINT hDMAChannel )
;
;   RETURN VALUE:
;      DWORD
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc _vmmwrapThunkVDMADProc, PUBLIC
                
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VDProc_Failure
        jmp     SHORT VDProc_CreateThunk

;==================
; Begin thunk code

VDMADThunk label   byte

        push    eax                             ; save for later
        push    eax
        push    ebx

        call    $
VDMADThunkCallAddr equ $-VDMADThunk

        add     esp, 8                          ; fixup for CDECL
        or      eax, eax
        pop     eax                             ; restore eax
        jz      SHORT VDMADThunk_Failure
        clc
        ret

VDMADThunk_Failure:
        stc
        ret

VDMADThunkSize    equ     $-VDMADThunk

;
;End thunk code
;==================

VDProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, VDMADThunk
        mov     edi, eax
        mov     ecx, VDMADThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, VDMADThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VDProc_Exit


VDProc_Failure:
        xor     eax, eax

VDProc_Exit:
        pop     ebp
        ret

EndProc _vmmwrapThunkVDMADProc

VxD_PAGEABLE_CODE_ENDS

END

