        page    60, 132

;******************************************************************************
	title   VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VMMWRAP - A 'C' wrapper for VMM services/control
;
;   Module:   Thunks
;
;   Version:  2.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE SegDefs.Inc
        .LIST

EXTRN Allocate_Procedure_Thunk:NEAR

VxD_PAGEABLE_CODE_SEG

;------------------------------------------------------------------------------
;
;   VXDIOPROC CDECL _vmmwrapThunkVXDIOProc( VXDIOPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for VxD I/O callbacks.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            UINT CDECL VXDIOProc( DWORD hVM, UINT uIOType,
;                                  UINT uPort, PCRS_32 pCRS,
;                                  UINT uData ) 
;
;   RETURN VALUE:
;      VXDIOPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc _vmmwrapThunkVXDIOProc, PUBLIC
        
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VIProc_Failure
        jmp     SHORT VIProc_CreateThunk

;==================
; Begin thunk code

VXDIOThunk label   byte

        push    eax                             ; uData
        push    ebp                             ; pCRS
        push    edx                             ; uPort
        push    ecx                             ; uIOType
        push    ebx                             ; hVM

        call    $
VXDIOThunkCallAddr equ $-VXDIOThunk
        add     esp, 20                         ; fixup for CDECL

        clc
        ret

VXDIOThunkSize    equ     $-VXDIOThunk

;
;End thunk code
;==================

VIProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, VXDIOThunk
        mov     edi, eax
        mov     ecx, VXDIOThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, VXDIOThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VIProc_Exit


VIProc_Failure:
        xor     eax, eax

VIProc_Exit:
        pop     ebp
        ret

EndProc _vmmwrapThunkVXDIOProc

VxD_PAGEABLE_CODE_ENDS

END
