        page    60, 132

;******************************************************************************
	title   VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VMMWRAP - A 'C' wrapper for VMM services/control
;
;   Module:   Thunks
;
;   Version:  2.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE SegDefs.Inc
        .LIST

EXTRN Allocate_Procedure_Thunk:NEAR

VxD_PAGEABLE_CODE_SEG

;------------------------------------------------------------------------------
;
;   IPFPROC CDECL vmmwrapThunkIPFProc( IPFPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for invalid page fault callbacks.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            BOOL CDECL IPFProc( DWORD hVM, PIPFDATA pIPFData )
;
;   RETURN VALUE:
;      IPFPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc _vmmwrapThunkIPFProc, PUBLIC
        
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT IPFProc_Failure
        jmp     SHORT IPFProc_CreateThunk

;==================
; Begin thunk code

IPFThunk label   byte

        push    eax
        push    edi                             ; pIPFData
        push    ebx                             ; hVM

        call    $
IPFThunkCallAddr equ $-IPFThunk
        add     esp, 8                          ; fixup for CDECL
        or      eax, eax
        pop     eax
        jnz     SHORT IPFThunk_Exit

        stc                                     ; signal not handled

IPFThunk_Exit:
        ret

IPFThunkSize    equ     $-IPFThunk

;
;End thunk code
;==================

IPFProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, IPFThunk
        mov     edi, eax
        mov     ecx, IPFThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, IPFThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT IPFProc_Exit


IPFProc_Failure:
        xor     eax, eax

IPFProc_Exit:
        pop     ebp
        ret

EndProc _vmmwrapThunkIPFProc

VxD_PAGEABLE_CODE_ENDS

END

