        page    60, 132

;******************************************************************************
	title   VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VMMWRAP - A 'C' wrapper for VMM services/control
;
;   Module:   Thunks
;
;   Version:  2.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE SegDefs.Inc
        .LIST

EXTRN Allocate_Procedure_Thunk:NEAR

VxD_PAGEABLE_CODE_SEG

;------------------------------------------------------------------------------
;
;   V86INTPROC CDECL vmmwrapThunkV86IntProc( V86INTPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for V86 interrupt handler callbacks.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            BOOL CDECL V86IntProc( DWORD hVM, UINT uInt, PCRS_32 pCRS )
;
;   RETURN VALUE:
;      V86INTPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc _vmmwrapThunkV86IntProc, PUBLIC
        
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VVIProc_Failure
        jmp     SHORT VVIProc_CreateThunk

;==================
; Begin thunk code

V86IntThunk label   byte

        push    eax
        push    ebp                             ; pCRS
        push    eax                             ; uInt
        push    ebx                             ; hVM

        call    $
V86IntThunkCallAddr equ $-V86IntThunk
        add     esp, 12                         ; fixup for CDECL
        or      eax, eax
        pop     eax                             ; restore eax
        jz      SHORT V86IntThunk_Failure
        clc
        ret

V86IntThunk_Failure:
        stc
        ret

V86IntThunkSize    equ     $-V86IntThunk

;
;End thunk code
;==================

VVIProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, V86IntThunk
        mov     edi, eax
        mov     ecx, V86IntThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, V86IntThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VVIProc_Exit


VVIProc_Failure:
        xor     eax, eax

VVIProc_Exit:
        pop     ebp
        ret

EndProc _vmmwrapThunkV86IntProc

VxD_PAGEABLE_CODE_ENDS

END

