        page    60, 132

;******************************************************************************
	title   VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VMMWRAP - A 'C' wrapper for VMM services/control
;
;   Module:   Thunks
;
;   Version:  2.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE SegDefs.Inc
        .LIST

EXTRN Allocate_Procedure_Thunk:NEAR

VxD_PAGEABLE_CODE_SEG

;------------------------------------------------------------------------------
;
;   V86PHPROC CDECL vmmwrapThunkV86PHProc( V86PHPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for VxD page fault callbacks.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            VOID CDECL V86PHProc( DWORD hVM, UINT uPage )
;
;   RETURN VALUE:
;      V86PHPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc _vmmwrapThunkV86PHProc, PUBLIC
        
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VHProc_Failure
        jmp     SHORT VHProc_CreateThunk

;==================
; Begin thunk code

V86PHThunk label   byte

        pushad
        push    eax                             ; uPage
        push    ebx                             ; hVM

        call    $
V86PHThunkCallAddr equ $-V86PHThunk
        add     esp, 8                          ; fixup for CDECL
        popad

        clc
        ret

V86PHThunkSize    equ     $-V86PHThunk

;
;End thunk code
;==================

VHProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, V86PHThunk
        mov     edi, eax
        mov     ecx, V86PHThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, V86PHThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VHProc_Exit


VHProc_Failure:
        xor     eax, eax

VHProc_Exit:
        pop     ebp
        ret

EndProc _vmmwrapThunkV86PHProc

VxD_PAGEABLE_CODE_ENDS

END

