        page    60, 132

;******************************************************************************
	title   VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VMMWRAP - A 'C' wrapper for VMM services/control
;
;   Module:   Thunks
;
;   Version:  2.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE SegDefs.Inc
        .LIST

EXTRN Allocate_Procedure_Thunk:NEAR

VxD_PAGEABLE_CODE_SEG

;------------------------------------------------------------------------------
;
;   VXDCBPROC CDECL vmmwrapThunkVxDCBProc( VXDCBPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for VxD generic event callbacks.
;
;   PARAMETERS:
;      VXDCBPROC pProc
;         pointer to callback procedure, must have the form:
;            VOID CDECL VxDCBProc( HVM hVM, DWORD dwRefData, PCRS_32 pCRS )
;
;   RETURN VALUE:
;      VXDCBPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc _vmmwrapThunkVxDCBProc, PUBLIC
        
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VCBProc_Failure
        jmp     SHORT VCBProc_CreateThunk

;==================
; Begin thunk code

VxDCBThunk label   byte

        push    ebp
        push    edx                             ; reference data
        push    ebx                             ; hVM

        call    $
VxDCBThunkCallAddr equ $-VxDCBThunk
        add     esp, 12                         ; fixup for CDECL
        ret

VxDCBThunkSize    equ     $-VxDCBThunk

;
;End thunk code
;==================

VCBProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, VxDCBThunk
        mov     edi, eax
        mov     ecx, VxDCBThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, VxDCBThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VCBProc_Exit


VCBProc_Failure:
        xor     eax, eax

VCBProc_Exit:
        pop     ebp
        ret

EndProc _vmmwrapThunkVxDCBProc

VxD_PAGEABLE_CODE_ENDS

END

