        page    60, 132

;******************************************************************************
	title	VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VMMWRAP - A 'C' wrapper for VMM services/control
;
;   Module:   Thunks
;
;   Version:  2.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE SegDefs.Inc
        INCLUDE Debug.Inc
        .LIST

VxD_PAGEABLE_DATA_SEG

;
; The "thunk table" is used to create headers for 'C' routines
; that are used with DMA trap handling, IO trap handling,
; API entry points, fault handlers, IRQ virtualization procedures, etc.
;

hThunkTable     dd      0
pThunkTable     dd      0
pCurrentThunk   dd      0

VxD_PAGEABLE_DATA_ENDS

VxD_PAGEABLE_CODE_SEG

;------------------------------------------------------------------------------
;
;   Allocate_Procedure_Thunk
;
;   DESCRIPTION:
;       Allocates a procedure thunk from the thunk table.
;
;   ENTRY:
;       NOTHING.
;
;   EXIT:
;       if Carry clear
;          success, EAX = Thunk address
;       else
;          failure
;
;   USES:
;       EAX, Flags
;
;------------------------------------------------------------------------------

Max_ThunkTablePages     equ     2
Size_ThunkTableEntry    equ     20h

BeginProc Allocate_Procedure_Thunk, PUBLIC
        
        push    edx
        cmp     hThunkTable, 0
        jne     SHORT APT_TablePresent

        VMMCall _PageAllocate, <Max_ThunkTablePages, PG_SYS, 0, 0, 0, 0, 0,\
                                PageFixed+PageZeroInit>
        or      eax, eax
        jz      SHORT APT_Failure
        mov     hThunkTable, eax
        mov     pThunkTable, edx
        mov     pCurrentThunk, edx

APT_TablePresent:
        mov     eax, pCurrentThunk
        sub     eax, pThunkTable
        mov     edx, Max_ThunkTablePages
        shl     edx, 12
        add     eax, Size_ThunkTableEntry
        cmp     eax, edx
        ja      SHORT APT_Failure
        add     eax, pThunkTable
        mov     pCurrentThunk, eax
        sub     eax, Size_ThunkTableEntry

APT_Success:
        clc
        jmp     SHORT APT_Exit

APT_Failure:
        Debug_Out "Unable to allocate procedure thunk!"

        stc

APT_Exit:
        pop     edx
        ret

EndProc Allocate_Procedure_Thunk


VxD_PAGEABLE_CODE_ENDS

END

