        page    60, 132

;******************************************************************************
	title	VMMWRAP - VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VMMWRAP - A 'C' wrapper for VMM services/control
;
;   Module:   VMMWRAP.ASM - Core code
;
;   Version:  2.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE SegDefs.Inc
        .LIST

EXTRN _alpVxDDispatchProcs:DWORD

VxD_LOCKED_CODE_SEG

;------------------------------------------------------------------------------
;
;   CDECL vmmwrapVxDControlProc
;
;   DESCRIPTION:
;       Dispatches VMM control messages to the appropriate handlers.
;
;   ENTRY:
;       EAX = Message
;       EBX = VM associated with message
;
;   EXIT:
;       Carry clear if no error (or if not handled by the VxD)
;       or set to indicate failure if the message can be failed.
;
;   USES:
;       All registers.
;
;------------------------------------------------------------------------------

BeginProc _vmmwrapVxDControlProc, PUBLIC

        push    esi
        lea     esi, _alpVxDDispatchProcs

VCP_Loop:
        cmp     dword ptr [esi], -1
        je      VCP_Not_Here
        cmp     eax, dword ptr [esi]
        je      SHORT VCP_Dispatch_It
        add     esi, 8
        jmp     SHORT VCP_Loop

VCP_Dispatch_It:
        cmp     eax, Sys_Critical_Init          ; Q: Sys_Critical_Init?
        jnz     SHORT VCP_Check_CmdTail         ;    N: check CmdTail frames
        mov     eax, [esi+4]                    ;    Y: special frame
        pop     esi
        push    ebp                             ; pointer to CRS
        push    esi                             ; command tail
        push    edx                             ; reference data
        push    ebx                             ; Sys VM handle
        call    eax
        add     esp, 16
        jmp     SHORT VCP_Dispatch_Complete

VCP_Check_CmdTail:
        cmp     eax, Device_Init                ; Q: Device_Init?
        jz      SHORT VCP_Dispatch_CmdTail      ;    Y: Ok
        cmp     eax, Init_Complete              ; Q: Init_Complete?
        jnz     SHORT VCP_Check_BPA             ;    N: Try again.

VCP_Dispatch_CmdTail:
        mov     eax, [esi+4]
        pop     esi
        push    ebp                             ; pointer to CRS
        push    esi                             ; command tail
        push    ebx                             ; Sys VM handle
        call    eax
        add     esp, 12
        jmp     SHORT VCP_Dispatch_Complete

VCP_Check_BPA:
        cmp     eax, Begin_PM_App               ; Q: Begin_PM_App?
        jnz     SHORT VCP_Check_SDF             ;    N: Try again.
        mov     eax, [esi+4]
        pop     esi
        push    ebp                             ; pointer to CRS
        push    edi                             ; ptr to Application CB
        push    edx                             ; flags
        push    ebx                             ; current VM handle
        call    eax
        add     esp, 16
        jmp     SHORT VCP_Dispatch_Complete

VCP_Check_SDF:
        cmp     eax, Set_Device_Focus           ; Q: Set_Device_Focus?
        jnz     SHORT VCP_Dispatch_Other        ;    N: Dispatch normal
        mov     eax, [esi+4]
        pop     esi
        push    edi                             ; associated VM
        push    esi                             ; flags
        push    edx                             ; device id to receive focus
        push    ebx                             ; VM handle
        call    eax
        add     esp, 16
        jmp     SHORT VCP_Dispatch_Complete

VCP_Dispatch_CheckParams:
        cmp     eax, Debug_Query
        je      SHORT VCP_Dispatch_CRS_Only
        cmp     eax, Sys_Critical_Exit
        jne     SHORT VCP_Dispatch_Other

VCP_Dispatch_SCE:
        mov     eax, [esi+4]
        pop     esi
        push    ebx
        call    eax
        add     esp, 4
        jmp     SHORT VCP_Exit_No_Fail

VCP_Dispatch_CRS_Only:
        mov     eax, [esi+4]
        pop     esi
        push    ebp                             ; pointer to CRS
        call    eax
        add     esp, 4
        jmp     SHORT VCP_Dispatch_Complete

VCP_Dispatch_Other:
        mov     eax, [esi+4]
        pop     esi     
        push    ebp                             ; pointer to CRS
        push    ebx                             ; current VM handle
        call    eax
        add     esp, 8

VCP_Dispatch_Complete:
        or      eax, eax
        jnz     SHORT VCP_Exit_No_Fail
        stc
        ret

VCP_Not_Here:
        pop     esi

VCP_Exit_No_Fail:
        clc
        ret

EndProc _vmmwrapVxDControlProc

VxD_LOCKED_CODE_ENDS

END

;---------------------------------------------------------------------------
;  End of File: vmmwrap.asm
;---------------------------------------------------------------------------

