//---------------------------------------------------------------------------
//
//  Module: wintypes.h
//
//  Purpose:
//     Provides "Win-type" definitions
//
//  Development Team:
//     Bryan A. Woodruff
//
//  History:   Date       Author      Comment
//              3/ 8/93   BryanW      Wrote it.
//
//---------------------------------------------------------------------------
//
//               Copyright (c) 1993 Woodruff Software Systems.
//                          All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef _WINTYPES_
#define _WINTYPES_

#ifndef WINVER

#define WINVER 0x30A

#endif

// stop the compiler griping...

#ifndef _MSC_VER
#define _MSC_VER 0
#endif

#if (_MSC_VER >= 700)
#pragma warning(disable:4103)
#endif

#pragma pack(1)         /* Assume byte packing throughout */

#if (_MSC_VER >= 700)
#pragma warning(disable:4103)
#endif

#ifdef __cplusplus
extern "C" {            /* Assume C declarations for C++ */
#endif   /* __cplusplus */

#define VOID               void

#define NEAR
#define FAR

#ifndef WATCOM_C386
#define STDCALL            _stdcall
#endif

#define CDECL              _cdecl

typedef int                BOOL ;

#define FALSE              0
#define TRUE               (~FALSE)

typedef char               CHAR ;
typedef short              SHORT ;
typedef unsigned char      BYTE, UCHAR ;
typedef unsigned short     WORD, USHORT ;
typedef unsigned long      DWORD ;
typedef long               LONG ;

typedef unsigned int       UINT ;
typedef unsigned long      ULONG ;

#ifndef NULL
#define NULL               0
#endif

typedef void               *PVOID, *LPVOID ;
typedef PVOID *            PPVOID ;

typedef char *             PSTR ;
typedef PSTR               PCHAR ;
typedef BYTE *             PBYTE ;
typedef BYTE *             LPBYTE ;
typedef int *              PINT ;
typedef UINT *             PUINT ;
typedef WORD *             PWORD ;
typedef PWORD              LPWORD ;    
typedef DWORD *            PDWORD ;
typedef PDWORD             LPDWORD ;
typedef UCHAR *            PUCHAR ;
typedef USHORT *           PUSHORT ;
typedef PUSHORT            LPUSHORT ;
typedef ULONG *            PULONG ;
typedef PULONG             LPULONG ;

#ifdef STRICT
typedef const void *            HANDLE;
#define DECLARE_HANDLE(name)    struct name##__ { int unused; }; \
                                typedef const struct name##__ * name
#else   /* STRICT */
typedef DWORD                   HANDLE;
#define DECLARE_HANDLE(name)    typedef DWORD name
#endif  /* !STRICT */


#define LONG               long

#define LOBYTE(w)          ((BYTE)(w))
#define HIBYTE(w)          ((BYTE)(((UINT)(w) >> 8) & 0xFF))
                            
#define LOWORD(l)          ((WORD)(DWORD)(l))
#define HIWORD(l)          ((WORD)((((DWORD)(l)) >> 16) & 0xFFFF))

#define MAKELONG(low, high) ((LONG)(((WORD)(low)) | (((DWORD)((WORD)(high))) << 16)))

typedef UINT WPARAM;
typedef LONG LPARAM;
typedef LONG LRESULT;

#if (_MSC_VER >= 700)
#define CALLBACK           _stdcall
#define WINAPI             _stdcall
#endif

DECLARE_HANDLE( HWND ) ;

DECLARE_HANDLE( HINSTANCE ) ;
DECLARE_HANDLE( HTASK ) ;

#ifdef __cplusplus
}                       /* End of extern "C" { */
#endif   /* __cplusplus */

#endif  // _WINTYPES_

//---------------------------------------------------------------------------
//  End of File: wintypes.h
//---------------------------------------------------------------------------

